<?php

namespace app\controllers;

use Yii;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\Response;
use yii\filters\VerbFilter;
use app\models\LoginForm;
use app\models\ContactForm;

class SiteController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'only' => ['logout'],
                'rules' => [
                    [
                        'actions' => ['logout'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
            'captcha' => [
                'class' => 'yii\captcha\CaptchaAction',
                'fixedVerifyCode' => YII_ENV_TEST ? 'testme' : null,
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */
    public function actionIndex()
    {
        
        $params=array();
        
        $API_ARRAY=array();
        $API_ARRAY[]= array('LABEL'=>'SSO User Information', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssouserinformation');
        $API_ARRAY[]= array('LABEL'=>'SSO Module Information', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssomoduleinformation');
        $API_ARRAY[]= array('LABEL'=>'SSO Module Permission Information', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssomodulepermission');
        $API_ARRAY[]= array('LABEL'=>'SSO Role Information', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssoroleinformation');
        $API_ARRAY[]= array('LABEL'=>'SSO User Access Right', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssouseraccessright');
        $API_ARRAY[]= array('LABEL'=>'SSO User Role', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssouserrole');
        $API_ARRAY[]= array('LABEL'=>'SSO User Token', 'URL'=>'http://demo.jumbolabs.com/join/sso/web/index.php/v1/ssousertoken ');
        
        
         
        $params['API_ARRAY']=$API_ARRAY;
        
        return $this->render('index',$params);
    }

    /**
     * Login action.
     *
     * @return Response|string
     */
    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            return $this->goBack();
        }

        $model->password = '';
        return $this->render('login', [
            'model' => $model,
        ]);
    }

    /**
     * Logout action.
     *
     * @return Response
     */
    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    /**
     * Displays contact page.
     *
     * @return Response|string
     */
    public function actionContact()
    {
        $model = new ContactForm();
        if ($model->load(Yii::$app->request->post()) && $model->contact(Yii::$app->params['adminEmail'])) {
            Yii::$app->session->setFlash('contactFormSubmitted');

            return $this->refresh();
        }
        return $this->render('contact', [
            'model' => $model,
        ]);
    }

    /**
     * Displays about page.
     *
     * @return string
     */
    public function actionAbout()
    {
        return $this->render('about');
    }
}
