<?php

namespace app\modules\admin\controllers;

use Yii;
use app\models\SsoModuleInformation;
use app\models\search\ssomoduleinformationsearch;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * ManagemoduleController implements the CRUD actions for SsoModuleInformation model.
 */
class ManagemoduleController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['index','view','create','delete','update'],
                        'allow' =>  Yii::$app->utility->isSuperAdmin(),
                    ],
                ],
                'denyCallback' => function ($rule, $action) {
                    if (Yii::$app->utility->isSuperAdmin() === false) {
                        return $action->controller->redirect(BASE_URL.'/site/login');
                    }
                    throw new ForbiddenHttpException("Forbidden access");
                }
            ],
            
        ];
    }

    /**
     * Lists all SsoModuleInformation models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ssomoduleinformationsearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single SsoModuleInformation model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new SsoModuleInformation model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    { 
        $model = new SsoModuleInformation();
        if(isset($_POST['SsoModuleInformation']['module_name'])) {
            $moduleTag = strtolower($_POST['SsoModuleInformation']['module_name']);
            $moduleTag = str_replace(" ","_",$moduleTag);            
            
            $model->module_tag =  $moduleTag;

            $moduleName = Ssomoduleinformation;
            $tableName  = SsoModuleInformationService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $model->module_uuid = $uuid;
            
            $salt = Yii::$app->utility->generatePasswordSalt();
            $model->module_hkey = $salt;

            $model->module_created_by =  "43343";
        }

        //echo "<pre>"; print_r(Yii::$app->request->post());
        
        //var_dump(Yii::$app->request->post()); die;
        
        $model->attributes = $_POST;
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->module_id]);
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /*$model->attributes = $_POST['SsoModuleInformation'];
        if ($model->validate()) {
            $model->save();				
            return $this->redirect(['view', 'id' => $model->module_id]);
        } else {
            $errors = $model->errors;
            echo "<pre>"; print_r($errors); die;
            return $this->render('create', [
                'model' => $model,
            ]);
        }*/

    /**
     * Updates an existing SsoModuleInformation model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->module_id]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing SsoModuleInformation model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the SsoModuleInformation model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return SsoModuleInformation the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = SsoModuleInformation::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
