<?php

namespace app\modules\admin\controllers;

use Yii;
use app\models\SsoRoleInformation;
use yii\filters\AccessControl;
use app\models\search\ssoroleinformationsearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;

/**
 * ManageroleController implements the CRUD actions for SsoRoleInformation model.
 */
class ManageroleController extends Controller
{
    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['index','view','create','delete','update'],
                        'allow' =>  Yii::$app->utility->isSuperAdmin(),
                    ],
                ],
                'denyCallback' => function ($rule, $action) {
                    if (Yii::$app->utility->isSuperAdmin() === false) {
                        return $action->controller->redirect(BASE_URL.'/site/login');
                    }
                    throw new ForbiddenHttpException("Forbidden access");
                }
            ],
            
        ];
    }

    /**
     * Lists all SsoRoleInformation models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ssoroleinformationsearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single SsoRoleInformation model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new SsoRoleInformation model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new SsoRoleInformation();
	if(isset($_POST['SsoRoleInformation']['role_name'])) {
            
            $moduleName = Ssoroleinformation;
            $tableName  = SsoRoleInformationService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $model->role_uuid = $uuid;
        }
        
        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->role_id]);
        } else {
            return $this->render('create', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Updates an existing SsoRoleInformation model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->role_id]);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Deletes an existing SsoRoleInformation model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the SsoRoleInformation model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return SsoRoleInformation the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = SsoRoleInformation::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
