<?php

namespace app\modules\admin\controllers;

use Yii;
use app\models\SsoUserInformation;
use app\models\search\ssouserinformationsearch;
use app\models\RbacUser;
use app\models\SsoUserRole;
use yii\filters\AccessControl;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\models\User;
use app\models\ApplicantInfo;

/**
 * ManageusersController implements the CRUD actions for SsoUserInformation model.
 */
class ManageusersController extends Controller
{

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['getcollegelist','index','view','create','delete','update','applicant'],
                        'allow' =>  Yii::$app->utility->isSuperAdmin() || Yii::$app->utility->isHelpDeskUser(),
                    ],

                ],
                'denyCallback' => function ($rule, $action) {
                    if (Yii::$app->utility->isSuperAdmin() === false) {
                        return $action->controller->redirect(BASE_URL.'/site/login');
                    }
                    throw new ForbiddenHttpException("Forbidden access");
                }
            ],

        ];
    }


    /**
     * Lists all SsoUserInformation models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new ssouserinformationsearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }


    /**
     * Lists all SsoUserInformation models.
     * @return mixed
     */
    public function actionApplicant()
    {
        $searchModel = new ssouserinformationsearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single SsoUserInformation model.
     * @param integer $id
     * @return mixed
     */
    public function actionView($id)
    {
      if(Yii::$app->utility->isSuperAdmin() === TRUE) {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
      } else {
        $model = $this->findModel($id);
        $registrationNumber = $model->registration_number;
        $applicantmodel = ApplicantInfo::find()
        ->where('applicant_info_registration_number = :applicant_info_registration_number', [':applicant_info_registration_number' => $registrationNumber])
        ->one();

        //echo "<pre>"; print_r($applicantmodel); die;
        return $this->render('view2', [
            'model' => $model,
            'data' => $applicantmodel
        ]);

      }
    }

    public function actionGetcollegelist(){
      if(isset($_POST['universityuuid'])){
        $uuid = $_POST['universityuuid'];
        $connection = Yii::$app->db;
        $sql = "select college_uuid, collegeid,collegename FROM dhe_legacy_college where affliationid =:id ;
              ";
        $query = $connection->createCommand($sql);
        $query->bindValue(':id', $uuid);
        $model = $query->queryAll();
        echo json_encode($model); die;
      }

    }
    /**
     * Creates a new SsoUserInformation model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {

        $model = new SsoUserInformation();
        if(isset($_POST['SsoUserInformation']['password_hash'])) {
            $moduleName = Ssouserinformation;
            $tableName  = SsoSsouserinformationService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $_POST['SsoUserInformation']['user_uuid'] = $uuid;

            $salt = Yii::$app->utility->generatePasswordSalt();
            $_POST['SsoUserInformation']['user_hkey'] = $salt;
            $passwordHash = hash_hmac('sha1', $_POST['SsoUserInformation']['password_hash'].$salt, PASSWORD_SECRET_KEY);


            $_POST['SsoUserInformation']['password_hash'] = $passwordHash;
            $model->password_hash =  $passwordHash;


            $model->attributes = $_POST['SsoUserInformation'];
            $connection = Yii::$app->db;
            $transaction = $connection->beginTransaction();
            if ($model->validate()) {
              $model->save();
              $res = $this->addUserInRbac($model->attributes);
              if($res == true){
                $params['user_uuid'] = $model->attributes['user_uuid'];

                $roleArray = explode("###",$_POST['SsoUserInformation']['user_role']);
                $roleId = $roleArray[0];
                $roleName = $roleArray[1];
                $userRole['user_uuid'] = $model->attributes['user_uuid'];
                $userRole['role_uuid'] = $roleId;
                $userRole['role_name'] = $roleName;
                $userRole = json_encode($userRole);
                $params['user_role'] = $userRole;
                $userRolePermission['role_permission'] = array();
                $userRolePermission['institute_module_permission'] = array();
                $userRolePermission['user_grant_permission'] = array();
                $userRolePermission['user_denied_permission'] = array();
                $userRolePermission['college_uuid'] = $_POST['SsoUserInformation']['college'];
                $userRolePermission = json_encode($userRolePermission);
                $params['user_role_permission'] = $userRolePermission;
                $userrole = $this->mapUserRole($params);
                if($userrole == true){
                    $transaction->commit();
                } else {
                  $transaction->rollBack();
                }
              } else{
                $transaction->rollBack();
              }
              return $this->redirect(['view', 'id' => $model->user_id]);
            }else{
              $errors = $model->errors;
              echo "<pre>"; print_r($errors); die;
            }
        }

       //echo "<pre>"; print_r($_POST); die;

        return $this->render('create', [
            'model' => $model,
        ]);

    }

    /**
     * Updates an existing SsoUserInformation model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        if(isset($_POST['SsoUserInformation']['password_hash'])) {

            $salt = Yii::$app->utility->generatePasswordSalt();
            $passwordHash = hash_hmac('sha1', $_POST['SsoUserInformation']['password_hash'].$salt, PASSWORD_SECRET_KEY);


            $_POST['SsoUserInformation']['password_hash'] = $passwordHash;
            $_POST['SsoUserInformation']['user_hkey'] = $salt;
            $model->password_hash =  $passwordHash;
            $model->user_hkey =  $salt;
        }
        if ($model->load($_POST) && $model->save()) {
          if(Yii::$app->utility->isHelpDeskUser() === TRUE){
            $this->updateUserDetail($_POST,$model->attributes['registration_number'],$model->user_id);
          }
          if(Yii::$app->utility->isSuperAdmin() === TRUE) {
            if(isset($_POST['SsoUserInformation']['college'])){
              $user_uuid = $model->attributes['user_uuid'];

              $roleArray = explode("###",$_POST['SsoUserInformation']['user_role']);
              $roleId = $roleArray[0];
              $roleName = $roleArray[1];
              $userRole['user_uuid'] = $model->attributes['user_uuid'];
              $userRole['role_uuid'] = $roleId;
              $userRole['role_name'] = $roleName;
              $userRole = json_encode($userRole);
              //$params['user_role'] = $userRole;
              $userRolePermission['role_permission'] = array();
              $userRolePermission['institute_module_permission'] = array();
              $userRolePermission['user_grant_permission'] = array();
              $userRolePermission['user_denied_permission'] = array();
              $userRolePermission['college_uuid'] = $_POST['SsoUserInformation']['college'];
              $userRolePermission = json_encode($userRolePermission);

              $rmodel = SsoUserRole::find()
              ->where('user_uuid = :user_uuid', [':user_uuid' => $user_uuid])
              ->one();

              $rmodel->user_role_permission = $userRolePermission;
              if ($rmodel->validate()) {
                $rmodel->save();
              }else{
                $errors = $rmodel->errors;
              }
            }
          }
            return $this->redirect(['view', 'id' => $model->user_id]);
        } else {
          $roles = array();
          $data = array();
          if(Yii::$app->utility->isHelpDeskUser() === TRUE){
            $registrationNumber = $model->registration_number;
            $connection = Yii::$app->db;
            $sql = "SELECT * FROM applicant_info WHERE applicant_info_registration_number = :applicant_info_registration_number ";
            $query = $connection->createCommand($sql);
            $query->bindValue(':applicant_info_registration_number', $registrationNumber);
            $data = $query->queryOne();
          }

          if(Yii::$app->utility->isSuperAdmin() === TRUE){
            $connection = Yii::$app->db;
            $status = 'ACTIVE';
            $sql = "select * FROM sso_role_information WHERE role_status = :status ";
            $query = $connection->createCommand($sql);
            $query->bindValue(':status', $status);
            $roles = $query->queryAll();
          }

            return $this->render('update', [
                'model' => $model,
                'roles' => $roles,
                'data' => $data
            ]);
        }
    }

    private function updateUserDetail($post, $registrationNumber, $user_id){
        if(isset($_POST['applicant'])) {

          $applicantmodel = ApplicantInfo::find()
          ->where('applicant_info_registration_number = :applicant_info_registration_number', [':applicant_info_registration_number' => $registrationNumber])
          ->one();

          //echo "<pre>"; print_r($applicantmodel->attributes); die;
          $applicantmodel->applicant_info_applicant_name = strtoupper($_POST['SsoUserInformation']['user_name']);
          $applicantmodel->applicant_info_mobile_number =  $_POST['SsoUserInformation']['user_mobile'];
          $applicantmodel->applicant_info_email_id = strtoupper($_POST['SsoUserInformation']['user_email']);
          $applicantmodel->applicant_info_father_name = strtoupper($_POST['applicant']['father_name']);
          $applicantmodel->applicant_info_mother_name = strtoupper($_POST['applicant']['mother_name']);
          $applicantmodel->applicant_info_date_of_birth = $_POST['applicant']['date_of_birth'];
          $applicantmodel->save();
          return $this->redirect(['view', 'id' => $user_id]);
        }
    }

    private function mapUserRole($params){
      $model = new SsoUserRole();
      $moduleName = Ssouserrolemapin;
      $tableName  = SsouserrolemapinService;
      $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
      $params['user_role_uuid'] = $uuid;
      $model->attributes = $params;
      if ($model->validate()) {
        $model->save();
        return true;
      }else{
        return false;
      }
    }

    private function addUserInRbac($post){
      $model = new RbacUser();
      $params['user_uuid'] = $post['user_uuid'];
      $params['user_name'] = $post['user_name'];
      $model->attributes = $params;
      if ($model->validate()) {
        $model->save();
        return true;
      }else{
        return false;
      }
    }



    /**
     * Deletes an existing SsoUserInformation model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     */
    public function actionDelete($id)
    {
        if(Yii::$app->utility->isHelpDeskUser() === TRUE){
          Yii::$app->session->setFlash('error', "Permission denied",true);
          return $this->redirect(['index']);
        }

        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the SsoUserInformation model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return SsoUserInformation the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = SsoUserInformation::findOne($id)) !== null) {
            return $model;
        } else {
            throw new NotFoundHttpException('The requested page does not exist.');
        }
    }
}
