<?php

namespace app\modules\api\v1\controllers;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\rest\ActiveController;
use yii\filters\AccessControl;
use yii\web\Response;
use yii\base\ErrorException;
use app\services\SsoUserInformationService;
use app\services\SsoModuleInformationService;
use app\services\SsoUserTokenService;

class AuthController extends \yii\rest\Controller
{

    private $rbacInfo=array();
    protected function verbs()
    {
        return [
            'changepassword'=>['POST'],
            'resetpassword'=>['POST'],
            'cmsforgotpassword'=>['POST'],
            'cmsresetpassword'=>['POST'],
            'forgotpassword' => ['POST'],
            'level1' => ['POST'],
            'level2' => ['POST'],
            'istokenactive' => ['POST'],
            'deletetoken'=>['POST'],
            'isvalidmobilenumberbyregistrtationnumber' => ['POST']
        ];
    }

    public function beforeAction($event)
    {
        // Allow from any origin
        if (isset($_SERVER['HTTP_ORIGIN'])) {
            header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
            header('Access-Control-Allow-Credentials: true');
            header('Access-Control-Max-Age: 86400');    // cache for 1 day
        }

        // Access-Control headers are received during OPTIONS requests
        if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
            if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD'])) {
                header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
            }

            if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS'])) {
                header("Access-Control-Allow-Headers:        {$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']}");
            }

            exit(0);
        }
        $this->enableCsrfValidation = false;
        $action = $event->id;
        if (isset($this->actions[$action])) {
            $verbs = $this->actions[$action];
        } elseif (isset($this->actions['*'])) {
            $verbs = $this->actions['*'];
        } else {
            return $event->isValid;
        }
        $verb = Yii::$app->getRequest()->getMethod();
        $allowed = array_map('strtoupper', $verbs);

        if (!in_array($verb, $allowed)) {
            http_response_code(401);
            $errorMsg="Method not allowed";
            $token=Yii::$app->request->headers['token'];
            $class=__FILE__;
            $method=__FUNCTION__;
            $model="SsoUserInformationService";
            $message=" ERROR $errorMsg at line ".__LINE__;
            Yii::$app->utility->logError($class, $method, $model, $token, $message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
        return true;
    }

    public function actionCmsresetpassword(){
     
        $params = $_POST;
        if (!isset($params) && empty($params)) {
            http_response_code(400);
            $errorMsg = "Bad request1";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } else {
          $result = "";         
          //if(!empty($params['user_uuid']) && !empty($params['reset_key']) && !empty($params['password'])){
            if (!isset($params['user_uuid']) || !isset($params['reset_key']) || !isset($params['password']) || !isset($params['module_tag']) || !isset($params['hmac'])) {
               
                http_response_code(400);
                $errorMsg = "Bad request";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } elseif (empty($params['user_uuid']) || empty($params['reset_key']) || empty($params['password']) || empty($params['module_tag']) || empty($params['hmac'])) {
               
                http_response_code(400);
                $errorMsg = "Bad request";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } else {     
                $moduleService = new SsoModuleInformationService();
                $moduleKey = $moduleService->getModuleHashByTag($_POST['module_tag']);
                $key = $moduleKey['module_hkey'];
                $string = $params['reset_key'].$params['user_uuid'].$params['password'].$params['module_tag'].$key;               $hmac = Yii::$app->utility->generateSha1($string);   


                if($hmac == $_POST['hmac']) {
                    $service = new SsoUserInformationService();
                    $result = $service->resetApplicantPassword($params['user_uuid'],  $params['reset_key'], $params['password']);
                    if($result === FALSE){
                        http_response_code(400);
                        $errorMsg = "INVALID_KEY";
                        echo json_encode(array("error" => $errorMsg));
                        exit;
                    } else{
                        $msg = "Password updated successfully";
                        http_response_code(200);
                        echo json_encode(array("response" => $msg));
                        exit;
                    }
                } else {
                    $errorMsg="Unauthorized";
                    $token=Yii::$app->request->headers['token'];
                    $class=__FILE__;
                    $method=__FUNCTION__;
                    $model="SsoUserInformationService";
                    $message=" ERROR $errorMsg at line ".__LINE__;
                    Yii::$app->utility->logError($class,$method,$model,$token,$message);
                    echo json_encode(array("error" => $errorMsg));
                    exit;   
                }            
            }
          }
      }
   
   public function actionForgotpassword(){
      $params = $_POST;
      if (!isset($params) && empty($params)) {
        http_response_code(400);
        $errorMsg = "Bad request1";
        echo json_encode(array("error" => $errorMsg));
        exit;
      }elseif (!isset($params['registration_number']) || !isset($params['mobile_number']) || !isset($params['password'])) {
        http_response_code(400);
        $errorMsg = "Bad request";
        echo json_encode(array("error" => $errorMsg));
        exit;
      } else {
        $service = new SsoUserInformationService();
        $result = $service->forgotpassword($params['registration_number'], $params['mobile_number'], $params['password']);
        http_response_code(200);
        if($result == TRUE){
          $msg = 'SUCCESS';
          echo json_encode(array("response" => $msg));
          exit;
        } else {
          $msg = 'ERROR';
          echo json_encode(array("error" => $msg));
          exit;
        }

      }
    }

    public function actionCmsforgotpassword(){
      
            $params=$_POST;
            if (!isset($params) && empty($params)) {
                http_response_code(400);
                $errorMsg = "Bad request1";
                echo json_encode(array("error" => $errorMsg));
                exit;

            }
        elseif (!isset($params['email'])) {
            http_response_code(400);
            $errorMsg = "Bad request";
            echo json_encode(array("error" => $errorMsg));
            exit;
        }  else {
          
            $moduleService = new SsoModuleInformationService();
          $moduleKey = $moduleService->getModuleHashByTag($_POST['module_tag']);
          
          $key = $moduleKey['module_hkey'];
           $string = $params['email'].$params['module_tag'].$key;  
           $hmac = Yii::$app->utility->generateSha1($string);
           


           if($hmac == $_POST['hmac']) {
            $service = new SsoUserInformationService();
            $result = $service->cms_forgotpassword($params);
        
            if(isset($result['error'])){
                $msg = $result['error'];
                echo json_encode(array("error" => $msg));
                exit;
            }elseif($result == TRUE){
                http_response_code(200);
                $msg = 'SUCCESS';
                echo json_encode(array("response" => $msg));
                exit;
            } else {
                $msg = 'ERROR';
                echo json_encode(array("error" => $msg));
                exit;
            }
        } else {
            http_response_code(401);
            $errorMsg = "Unauthorized";
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
      }
    }
    public function actionIsvalidmobilenumberbyregistrtationnumber(){
      $params = $_POST;
      if (!isset($params) && empty($params)) {
        http_response_code(400);
        $errorMsg = "Bad request1";
        echo json_encode(array("error" => $errorMsg));
        exit;
      }elseif (!isset($params['registration_number']) || !isset($params['mobile_number'])) {
          http_response_code(400);
          $errorMsg = "Bad request2";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } else {
        $service = new SsoUserInformationService();
        $result = $service->isValidRegistrationNumberAndMobileNumber($params['registration_number'], $params['mobile_number']);
        http_response_code(200);
        $msg = "success";
        http_response_code(200);
        echo json_encode(array("response" => $result));
        exit;
      }
    }

    public function actionResetpassword(){
      $params = $_POST;
      if (!isset($params) && empty($params)) {
          http_response_code(400);
          $errorMsg = "Bad request1";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } elseif (!isset($params['email']) || !isset($params['username']) || !isset($params['registration_number'])) {
          http_response_code(400);
          $errorMsg = "Bad request2";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } else {
        //echo "<pre>"; print_r($_POST); die;
        if(!empty($params['email'])){ //echo $params['email']; die;
           $service = new SsoUserInformationService();
            $result = $service->isValidEmail($params['email'], $params['reason']);
        }
        if(!empty($params['username'])){
          $service = new SsoUserInformationService();
            $result = $service->isValidUserName($params['username'], $params['reason']);
        }
        if(!empty($params['registration_number'])){
          $service = new SsoUserInformationService();
            $result = $service->isValidRegistration($params['registration_number'],  $params['reason']);
        }
        if($result === TRUE){
          http_response_code(200);
          $msg = "success";
          http_response_code(200);
          echo json_encode(array("response" => $msg));
          exit;
        } else {
          http_response_code(400);
          $errorMsg = "ERROR";
          echo json_encode(array("error" => $errorMsg));
          exit;
        }

      }
    }


    public function actionChangepassword(){
      $params = $_POST;
      if (!isset($params) && empty($params)) {
          http_response_code(400);
          $errorMsg = "Bad request1";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } elseif (!isset($params['user_uuid']) || !isset($params['old_password']) || !isset($params['new_password'])) {
          http_response_code(400);
          $errorMsg = "Bad request2";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } else if(empty($params['user_uuid']) || empty($params['old_password']) || empty($params['new_password'])) {
          http_response_code(400);
          $errorMsg = "Bad request2";
          echo json_encode(array("error" => $errorMsg));
          exit;
      } else {

        $service = new SsoUserInformationService();
        $result = $service->changePasswordForChangePassword($params);

        if($result === TRUE){
          http_response_code(200);
          $msg = "success";
          http_response_code(200);
          echo json_encode(array("response" => $msg));
          exit;
        } else {
          http_response_code(400);
          $errorMsg = $result;
          echo json_encode(array("error" => $errorMsg));
          exit;
        }

      }
    }


    public function actionLevel1()
    {

        //echo "<pre>"; print_r($_POST); die('in api');
        $params = $_POST;
		
        if (!isset($params) && empty($params)) {
            http_response_code(400);
            $errorMsg = "Bad request2";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } elseif (!isset($params['email']) || !isset($params['password']) || !isset($params['module_tag']) || !isset($params['hmac'])) {
            http_response_code(400);
            $errorMsg = "Bad request1";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } elseif (empty($params['email']) || empty($params['password']) || empty($params['module_tag']) || empty($params['hmac'])) {
            http_response_code(400);
            $errorMsg = "Required fields are missing";
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
        // elseif (!filter_var($params['email'], FILTER_VALIDATE_EMAIL)) {
        //     http_response_code(400);
        //     $errorMsg = "Invalid Email";
        //     echo json_encode(array("error" => $errorMsg));
        //     exit;
        // }
        else {
            $moduleTag = $params['module_tag'];
            $service = new SsoModuleInformationService();
            $dataProvider = $service->getModuleHashByTag($moduleTag);
            if (isset($dataProvider['module_hkey'])) {
                $hkey = $dataProvider['module_hkey'];
                $moduleHmac = $params['hmac'];
                $postValues = $params['email'].$params['password'].$params['module_tag'].$hkey;
                $hmac = Yii::$app->utility->generateSha1($postValues);
                if ($moduleHmac == $hmac) {
                    $service = new SsoUserInformationService();

                    $result = $service->isValidCredentials($params['email'], $params['password']);
                   //
                    if ($result === false) {
                        http_response_code(400);
                        $errorMsg = "Not found";
                        echo json_encode(array("error" => $errorMsg));
                        exit;
                    } else {
                        if ($result['user_status'] == 'INACTIVE') {
                            http_response_code(400);
                            $errorMsg = "Inactive Account";
                            echo json_encode(array("error" => $errorMsg));
                            exit;
                        } else {
                       // echo "<pre>"; print_r($result); die;
                            $tokenInfo = array();
                            $tokenInfo['token'] = $result['token'];
                            $tokenInfo['user_uuid'] = $result['user_uuid'];
                            $tokenInfo['user_name'] = $result['user_name'];
                            $data = Yii::$app->utility->getRBAC($result['token']);
                            $tokenInfo['user_role'] = $data['user_role'];
                            $tokenInfo['user_permission'] = $data['user_role_permission'];


                            http_response_code(200);
                            echo json_encode(array("response" => $tokenInfo));
                        }
                    }
                } else {
                    http_response_code(401);
                    $errorMsg = "Unauthorized";
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            } else {
                
                http_response_code(401);
                $errorMsg = "Unauthorized";
                echo json_encode(array("error" => $errorMsg));
                exit;
            }
        }
    }


    public function actionLevel2()
    {
        $params = $_POST;
        if (!isset($params) && empty($params)) {
            http_response_code(400);
            $errorMsg = "Bad request";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } elseif (!isset($params['registration_number']) || !isset($params['password']) || !isset($params['module_tag']) || !isset($params['hmac'])) {
            http_response_code(400);
            $errorMsg = "Bad request";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } elseif (empty($params['registration_number']) || empty($params['password']) || empty($params['module_tag']) || empty($params['hmac'])) {
            http_response_code(400);
            $errorMsg = "Required fields are missing";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } else {
            $moduleTag = $params['module_tag'];
            $service = new SsoModuleInformationService();
            $dataProvider = $service->getModuleHashByTag($moduleTag);
            if (isset($dataProvider['module_hkey'])) {
                $hkey = $dataProvider['module_hkey'];
                $moduleHmac = $params['hmac'];
                $postValues = $params['registration_number'].$params['password'].$params['module_tag'].$hkey;
                $hmac = Yii::$app->utility->generateSha1($postValues);
                if ($moduleHmac == $hmac) {
                    $service = new SsoUserInformationService();
                    $result = $service->isValidRegistrationNumberAndPassword($params['registration_number'], $params['password']);
                    if ($result === false) {
                        http_response_code(400);
                        $errorMsg = "Not found";
                        echo json_encode(array("error" => $errorMsg));
                        exit;
                    } else {
                        if ($result['user_status'] == 'INACTIVE') {
                            http_response_code(400);
                            $errorMsg = "Inactive Account";
                            echo json_encode(array("error" => $errorMsg));
                            exit;
                        } else {
                            $tokenInfo = array();
                            $tokenInfo['token'] = $result['token'];
                            $tokenInfo['user_uuid'] = $result['user_uuid'];
                            $tokenInfo['user_name'] = $result['user_name'];
							$tokenInfo['aadhar_number'] = $result['aadhar_number'];

                            http_response_code(200);
                            $errorMsg = "Inactive Account";
                            echo json_encode(array("response" => $tokenInfo));
                        }
                    }
                } else {
                    http_response_code(401);
                    $errorMsg = "Unauthorized";
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            } else {
                http_response_code(401);
                $errorMsg = "Unauthorized";
                echo json_encode(array("error" => $errorMsg));
                exit;
            }
        }
    }

    /**
    *Function to check token is active or not
    *@param POST String $_POST['token']
    *@param POST String $_POST['module_tag']
    *@param POST String $_POST['hmac']
    *$_POST['hmac'] = hash_hmac(algo (sha1), String ($_POST['token'].$_POST['module_tag']), secret (module hkey))
    *@return String $response [error_code or success_code]
    */
    public function actionIstokenactive()
    {
        $params = $_POST;
        if (!isset($params) && empty($params)) {
            http_response_code(400);
            $errorMsg = "Bad request";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } elseif (Yii::$app->utility->isValidJson($params) === false) {
            http_response_code(400);
            $errorMsg = "Bad request";
            echo json_encode(array("error" => $errorMsg));
            exit;
        } else {
            if (!isset($params['token']) || !isset($params['module_tag']) || !isset($params['hmac'])) {
                http_response_code(400);
                $errorMsg = "Bad request";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } elseif (empty($params['token']) || empty($params['module_tag']) || empty($params['hmac'])) {
                http_response_code(400);
                $errorMsg = "Bad request";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } else {
                $moduleTag = $params['module_tag'];
                $service = new SsoModuleInformationService();
                $dataProvider = $service->getModuleHashByTag($moduleTag);
                if (isset($dataProvider['module_hkey'])) {
                    $hkey = $dataProvider['module_hkey'];
                    $moduleHmac = $params['hmac'];
                    $postValues = $params['email'].$params['password'].$params['module_tag'].$hkey;
                    $hmac = Yii::$app->utility->generateSha1($postValues);
                    if ($moduleHmac == $hmac) {
                        $token = $params['token'];
                        $service = new SsoUserTokenService();
                        $response = $service->isTokenActive($token);
                        if ($response === true) {
                            http_response_code(200);
                            $successMsg = "OK";
                            echo json_encode(array("response" => $successMsg));
                            exit;
                        } else {
                            http_response_code(401);
                            $errorMsg = "Unauthorized";
                            echo json_encode(array("error" => $errorMsg));
                            exit;
                        }
                    } else {
                        http_response_code(401);
                        $errorMsg = "Unauthorized";
                        echo json_encode(array("error" => $errorMsg));
                        exit;
                    }
                } else {
                    http_response_code(401);
                    $errorMsg = "Unauthorized";
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            }
        }
    }

    /**
    *Function to delete token
    *@param POST String $_POST['token']
    *@param POST String $_POST['module_tag']
    *@param POST String $_POST['hmac']
    *$_POST['hmac'] = hash_hmac(algo (sha1), String ($_POST['token'].$_POST['module_tag']), secret (module hkey))
    *@return String $response [error_code or success_code]
    */
    public function actionDeletetoken(){
        $params = $_POST;

		//echo "<pre>"; print_r($_POST); exit;
        if (!isset($params) && empty($params)) {
            http_response_code(400);
                $errorMsg = "Bad request1";
                echo json_encode(array("error" => $errorMsg));
                exit;
        } elseif (Yii::$app->utility->isValidJson($params) === false) {
			//echo "<pre>"; print_r($_POST); exit;
            http_response_code(400);
                $errorMsg = "Bad request2";
                echo json_encode(array("error" => $errorMsg));
                exit;
        } else {
            //$params=(array)json_decode($params);
            //echo "<pre>"; print_r($_POST); die;
            if (!isset($params['token']) && !isset($params['module_tag']) && !isset($params['hmac'])) {
                http_response_code(400);
                $errorMsg = "Bad request3";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } elseif (empty($params['token']) && empty($params['module_tag']) && empty($params['hmac'])) {
                http_response_code(400);
                $errorMsg = "Bad request4";
                echo json_encode(array("error" => $errorMsg));
                exit;
            } else {
                $moduleTag = $params['module_tag'];
                $service = new SsoModuleInformationService();
                $dataProvider = $service->getModuleHashByTag($moduleTag);
                if (isset($dataProvider['module_hkey'])) {
                    $hkey = $dataProvider['module_hkey'];
                    $moduleHmac = $params['hmac'];
                    $postValues = $params['email'].$params['password'].$params['module_tag'].$hkey;
                    $hmac = Yii::$app->utility->generateSha1($postValues);
                    if ($moduleHmac == $hmac) {
                        $token = $params['token'];
                        $service = new SsoUserTokenService();
                        $response = $service->deleteToken($token);
                        if ($response === true) {
                            http_response_code(200);
                            $successMsg = "OK";
                            echo json_encode(array("response" => $successMsg));
                            exit;
                        } else {
                            http_response_code(401);
                            $errorMsg = "Unauthorized";
                            echo json_encode(array("error" => $errorMsg));
                            exit;
                        }
                    } else {
                        http_response_code(401);
                        $errorMsg = "Unauthorized";
                        echo json_encode(array("error" => $errorMsg));
                        exit;
                    }
                } else {
                    http_response_code(401);
                    $errorMsg = "Unauthorized";
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            }
        }
    }

}
