<?php

namespace app\modules\api\v1\controllers;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\rest\ActiveController;
use yii\filters\AccessControl;
use yii\web\Response;
use yii\base\ErrorException;
use app\services\SsoUserAccessrightService;

class SsouseraccessrightController extends \yii\rest\Controller {
	
	private $rbacInfo=array();
    protected function verbs() {
        return [
            'index' =>  ['GET'],
            'getbyuuid' => ['POST'],
			'create' => ['POST'],
            'update' => ['POST'],
            'delete' => ['POST'],
        ];
    }
    
    public function beforeAction($event) { 
    	// Allow from any origin
	if (isset($_SERVER['HTTP_ORIGIN'])) {
		header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
		header('Access-Control-Allow-Credentials: true');
		header('Access-Control-Max-Age: 86400');    // cache for 1 day
	}

	// Access-Control headers are received during OPTIONS requests
	if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
		if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD']))
		    header("Access-Control-Allow-Methods: GET, POST, OPTIONS");         

		if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']))
		    header("Access-Control-Allow-Headers:        {$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']}");

		exit(0);
	}
       $this->enableCsrfValidation = false;
       $headers = Yii::$app->request->headers; 
       if(!$headers->has('token')) {
            http_response_code(401);
			$errorMsg="Unauthorized Access";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserAccessrightService";
		    $message=" ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logError($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
       } 
	   
		else {
			if( Yii::$app->utility->isValidToken(Yii::$app->request->headers['token']) ) {
				$token = Yii::$app->request->headers['token'];
				$this->rbacInfo= Yii::$app->utility->getRBAC($token);
	   		}
           $action = $event->id;
           if (isset($this->actions[$action])) {
               $verbs = $this->actions[$action];
           } elseif (isset($this->actions['*'])) {
               $verbs = $this->actions['*'];
           } else {
               return $event->isValid;
           }
           $verb = Yii::$app->getRequest()->getMethod();       
           $allowed = array_map('strtoupper', $verbs);

           if (!in_array($verb, $allowed)) {
               http_response_code(401);
			    $errorMsg="Method not allowed";
				$token=Yii::$app->request->headers['token'];
				$class=__FILE__;
				$method=__FUNCTION__;
				$model="SsoUserAccessrightService";
				$message=" ERROR $errorMsg at line ".__LINE__;
				Yii::$app->utility->logError($class,$method,$model,$token,$message);
               echo json_encode(array("error" => $errorMsg));
               exit;
           }        
           return true;
       }
    } 


    public function actionIndex() {
        
        $accessright_is_deleted="NO";
        $model = SsoUserAccessrightService::find()
                    ->where('accessright_is_deleted = :accessright_is_deleted', [':accessright_is_deleted' => $accessright_is_deleted])
					->asArray()
				    ->all();
        
        echo json_encode(array("response" => $model));
        exit;
    }

	 public function actionGetbyuuid() {
        
         try {
            $accessright_uuid = $_POST['accessright_uuid'];

            $model = SsoUserAccessrightService::find()
                    ->where('accessright_uuid = :accessright_uuid', [':accessright_uuid' => $accessright_uuid])
					->one();
            if(!$model){
                echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                exit; 
            }          		

			$row= $model->attributes;
            return json_encode(array("response" => $row));        
        
        } 
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserAccessrightService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }

    public function actionCreate() {   
        
        try {
            $model = new SsoUserAccessrightService();
            
            if(!$model){
				$errorMsg="Invalid Method or parameters. Please try again";
				$token=Yii::$app->request->headers['token'];
				$class=__FILE__;
				$method=__FUNCTION__;
				$model="SsoUserAccessrightService";
				$message=" ERROR $errorMsg at line ".__LINE__;
				Yii::$app->utility->logError($class,$method,$model,$token,$message);
                echo json_encode(array("error" => $errorMsg));
                exit; 
            }
	
			$moduleName = Ssouseraccessright;
            $tableName  = SsoUserAccessrightService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $_POST['accessright_uuid'] = $uuid;

            $model->attributes = $_POST;
            if ($model->validate()) {
                $model->save();
                echo json_encode(array("response" => $model));
                exit;
            } else {
                $errors = $model->errors;
                echo json_encode(array("error" => $errors));
                exit;
            }
            
        } 
        catch (ErrorException $e) {
			$errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserAccessrightService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
        
    }

    public function actionUpdate() {
        try {
            $accessright_uuid = $_POST['accessright_uuid'];

            $model = SsoUserAccessrightService::find()
                    ->where('accessright_uuid = :accessright_uuid', [':accessright_uuid' => $accessright_uuid])
					->one();
            if(!$model){
                echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                exit; 
            }

            $model->attributes = $_POST;
            if ($model->validate()) {
                $model->save();				
				$row= $model->attributes;
                return json_encode(array("response" => $row));
                exit;
            } else {
                $errors = $model->errors;
                echo json_encode(array("error" => $errors));
                exit;
            }
            
        } 
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserAccessrightService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }
    
    
    public function actionDelete() {
        try {
            $accessright_uuid = $_POST['accessright_uuid'];

            $model = SsoUserAccessrightService::find()
                    ->where('accessright_uuid = :accessright_uuid', [':accessright_uuid' => $accessright_uuid])
         			->one();
            if(!$model){
                echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                exit; 
            }

            $model->accessright_is_deleted = "YES";
            if ($model->validate()) {
                $model->save();
				$row= $model->attributes;
                echo json_encode(array("response" => $row));
                exit;
            } else {
                $errors = $model->errors;
                echo json_encode(array("error" => $errors));
                exit;
            }
            
        } 
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserAccessrightService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }

}
