<?php

namespace app\modules\api\v1\controllers;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\rest\ActiveController;
use yii\filters\AccessControl;
use yii\web\Response;
use yii\base\ErrorException;
use app\services\SsoUserInformationService;
use app\services\SsoUserTokenService;
use app\services\SsoModuleInformationService;

use app\models\SsoUserInformation;
use app\models\search\ssouserinformationsearch;
use app\models\RbacUser;
use app\models\SsoUserRole;

class SsouserinformationController extends \yii\rest\Controller {

	private $rbacInfo=array();
    protected function verbs() {
        return [
            'index' =>  ['GET'],
            'getbyuuid' => ['POST'],
			'create' => ['POST'],
            'update' => ['POST'],
            'delete' => ['POST'],
            'createauthuser' => ['POST']
        ];
    }

    public function beforeAction($event) {
    	// Allow from any origin
	if (isset($_SERVER['HTTP_ORIGIN'])) {
		header("Access-Control-Allow-Origin: {$_SERVER['HTTP_ORIGIN']}");
		header('Access-Control-Allow-Credentials: true');
		header('Access-Control-Max-Age: 86400');    // cache for 1 day
	}

	// Access-Control headers are received during OPTIONS requests
	if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
		if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_METHOD']))
		    header("Access-Control-Allow-Methods: GET, POST, OPTIONS");

		if (isset($_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']))
		    header("Access-Control-Allow-Headers:        {$_SERVER['HTTP_ACCESS_CONTROL_REQUEST_HEADERS']}");

		exit(0);
	}
       $this->enableCsrfValidation = false;
       $headers = Yii::$app->request->headers;
       if(!$headers->has('token')) {
            http_response_code(401);
			$errorMsg="Unauthorized Access";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserInformationService";
		    $message=" ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logError($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
       }

		else {
			if( Yii::$app->utility->isValidToken(Yii::$app->request->headers['token']) ) {
				$token = Yii::$app->request->headers['token'];
				$this->rbacInfo= Yii::$app->utility->getRBAC($token);
	   		}
           $action = $event->id;
           if (isset($this->actions[$action])) {
               $verbs = $this->actions[$action];
           } elseif (isset($this->actions['*'])) {
               $verbs = $this->actions['*'];
           } else {
               return $event->isValid;
           }
           $verb = Yii::$app->getRequest()->getMethod();
           $allowed = array_map('strtoupper', $verbs);

           if (!in_array($verb, $allowed)) {
               http_response_code(401);
			    $errorMsg="Method not allowed";
				$token=Yii::$app->request->headers['token'];
				$class=__FILE__;
				$method=__FUNCTION__;
				$model="SsoUserInformationService";
				$message=" ERROR $errorMsg at line ".__LINE__;
				Yii::$app->utility->logError($class,$method,$model,$token,$message);
               echo json_encode(array("error" => $errorMsg));
               exit;
           }
           return true;
       }
    }


    public function actionIndex() {

        $user_is_deleted="NO";
        $model = SsoUserInformationService::find()
                    ->where('user_is_deleted = :user_is_deleted', [':user_is_deleted' => $user_is_deleted])
					->asArray()
				    ->all();

        echo json_encode(array("response" => $model));
        exit;
    }

	 public function actionGetbyuuid() {

         try {
            $user_uuid = $_POST['user_uuid'];

            $model = SsoUserInformationService::find()
                    ->where('user_uuid = :user_uuid', [':user_uuid' => $user_uuid])
					->one();
            if(!$model){
                echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                exit;
            }

			$row= $model->attributes;
            return json_encode(array("response" => $row));

        }
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserInformationService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }

    public function actionCreate() {

        try {
					//echo "<pre>"; print_r($_POST); die;
            $model = new SsoUserInformationService();

            if(!$model){
							$errorMsg="Invalid Method or parameters. Please try again";
							$token=Yii::$app->request->headers['token'];
							$class=__FILE__;
							$method=__FUNCTION__;
							$model="SsoUserInformationService";
							$message=" ERROR $errorMsg at line ".__LINE__;
							Yii::$app->utility->logError($class,$method,$model,$token,$message);
                echo json_encode(array("error" => $errorMsg));
                exit;
            }

						$moduleName = Ssouserinformation;
            $tableName  = SsoUserInformationService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $_POST['user_uuid'] = $uuid;

            if(isset($_POST['module_tag']) && !empty($_POST['module_tag']) && !empty($_POST['aadhar_number'])) {
                $moduleService = new SsoModuleInformationService();
                $moduleKey = $moduleService->getModuleHashByTag($_POST['module_tag']);
                $key = $moduleKey['module_hkey'];
                $string = $_POST['user_name'].$_POST['password_hash'].$_POST['module_tag'].$key;
                $hmac = Yii::$app->utility->generateSha1($string);

                $salt = Yii::$app->utility->generatePasswordSalt();
                $_POST['user_hkey'] = $salt;
                $passwordHash = hash_hmac('sha1', $_POST['password_hash'].$salt, PASSWORD_SECRET_KEY);
                $_POST['password_hash'] = $passwordHash;
								//$_POST['aadhar_number'] = $_POST['aadhar_number'];

                if($hmac == $_POST['hmac']) {
                    $model->attributes = $_POST;
                    if ($model->validate()) {
                        $model->save();
                        $response = array();
                        $response['user_uuid'] = $model->attributes['user_uuid'];
                        if(LOGIN_AFTER_REGISTRATION === TRUE){
                            $service = new SsoUserTokenService();
                            $token = $service->saveUserToken($model->attributes['user_uuid']);
                            $response['token'] = $token;
                        }
                        echo json_encode(array("response" => $response));
                        exit;
                    } else {
                        $errors = $model->errors;
                        echo json_encode(array("error" => $errors));
                        exit;
                    }
                } else {
                    $errorMsg="Invalid hmac";
                    $token=Yii::$app->request->headers['token'];
                    $class=__FILE__;
                    $method=__FUNCTION__;
                    $model="SsoUserInformationService";
                    $message=" ERROR $errorMsg at line ".__LINE__;
                    Yii::$app->utility->logError($class,$method,$model,$token,$message);
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            } else {
                $errorMsg="Module tag missing";
				$token=Yii::$app->request->headers['token'];
				$class=__FILE__;
				$method=__FUNCTION__;
				$model="SsoUserInformationService";
				$message=" ERROR $errorMsg at line ".__LINE__;
				Yii::$app->utility->logError($class,$method,$model,$token,$message);
                echo json_encode(array("error" => $errorMsg));
                exit;
            }

        }
        catch (ErrorException $e) {
			$errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserInformationService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }

    }

    public function actionUpdate() {
        try {
            if(isset($_POST['module_tag']) && !empty($_POST['module_tag'])) {
                $user_uuid = trim($_POST['user_uuid']);
                $moduleService = new SsoModuleInformationService();
                $moduleKey = $moduleService->getModuleHashByTag($_POST['module_tag']);
                $key = $moduleKey['module_hkey'];
                $string = $_POST['user_uuid'].$_POST['registration_number'].$key;
                $hmac = Yii::$app->utility->generateSha1($string);
                if($hmac == $_POST['hmac']) {
                    if(isset($_POST['registration_number']) && !empty($_POST['registration_number'])){
                        $_POST['user_status'] = 'ACTIVE';
                    }
                    $model = SsoUserInformationService::find()
                            ->where('trim(user_uuid) = :user_uuid', [':user_uuid' => $user_uuid])
                    ->one();

                    if(!$model){
                        echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                        exit;
                    }


                    $model->attributes = $_POST;
                    if ($model->validate()) {
                        $model->save();
                        $row= $model->attributes;
                        return json_encode(array("response" => $row));
                        exit;
                    } else {
                        $errors = $model->errors;
                        echo json_encode(array("error" => $errors));
                        exit;
                    }
                } else {
                    $errorMsg="Invalid hmac";
                    $token=Yii::$app->request->headers['token'];
                    $class=__FILE__;
                    $method=__FUNCTION__;
                    $model="SsoUserInformationService";
                    $message=" ERROR $errorMsg at line ".__LINE__;
                    Yii::$app->utility->logError($class,$method,$model,$token,$message);
                    echo json_encode(array("error" => $errorMsg));
                    exit;
                }
            }
        }
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserInformationService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }


    public function actionDelete() {
        try {
            $user_uuid = $_POST['user_uuid'];

            $model = SsoUserInformationService::find()
                    ->where('user_uuid = :user_uuid', [':user_uuid' => $user_uuid])
         			->one();
            if(!$model){
                echo json_encode(array("error" => "Invalid Method or parameters. Please try again"));
                exit;
            }

            $model->user_is_deleted = "YES";
            if ($model->validate()) {
                $model->save();
				$row= $model->attributes;
                echo json_encode(array("response" => $row));
                exit;
            } else {
                $errors = $model->errors;
                echo json_encode(array("error" => $errors));
                exit;
            }

        }
        catch (ErrorException $e) {
            $errorMsg="Invalid Method. Please try again";
			$token=Yii::$app->request->headers['token'];
			$class=__FILE__;
			$method=__FUNCTION__;
			$model="SsoUserInformationService";
			$message= $e->getMessage()." ERROR $errorMsg at line ".__LINE__;
			Yii::$app->utility->logException($class,$method,$model,$token,$message);
            echo json_encode(array("error" => $errorMsg));
            exit;
        }
    }
	
	private function addUserInRbac($post){
      $model = new RbacUser();
      $params['user_uuid'] = $post['user_uuid'];
      $params['user_name'] = $post['user_name'];
      $model->attributes = $params;
      if ($model->validate()) {
        $model->save();
        return true;
      }else{
        return false;
      }
    }

    private function mapUserRole($params){
      $model = new SsoUserRole();
      $moduleName = Ssouserrolemapin;
      $tableName  = SsouserrolemapinService;
      $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
      $params['user_role_uuid'] = $uuid;
      $model->attributes = $params;
      if ($model->validate()) {
        $model->save();
        return true;
      }else{
        return false;
      }
    }

    public function actionCreateauthuser()
    {
        echo "<pre>"; print_r($_POST); die;
      
        $model = new SsoUserInformation();
        $_POST['SsoUserInformation'] = $_POST;
        if(isset($_POST['SsoUserInformation']['password_hash'])) {
            
            $moduleName = Ssouserinformation;
            $tableName  = SsoSsouserinformationService;
            $uuid = Yii::$app->utility->generateGuuid($moduleName,$tableName);
            $_POST['SsoUserInformation']['user_uuid'] = $uuid;

            $salt = Yii::$app->utility->generatePasswordSalt();
            $_POST['SsoUserInformation']['user_hkey'] = $salt;
            $passwordHash = hash_hmac('sha1', $_POST['SsoUserInformation']['password_hash'].$salt, PASSWORD_SECRET_KEY);


            $_POST['SsoUserInformation']['password_hash'] = $passwordHash;
            $model->password_hash =  $passwordHash;

            $model->attributes = $_POST['SsoUserInformation'];
            $connection = Yii::$app->db;
            $transaction = $connection->beginTransaction();

            if ($model->validate()) {

              $model->save();
              $res = $this->addUserInRbac($model->attributes);

              if($res == true){
                $params['user_uuid'] = $model->attributes['user_uuid'];
                 
                $roleArray = explode("###",$_POST['SsoUserInformation']['user_role']);
                $roleId = $roleArray[0];
                $roleName = $_POST['SsoUserInformation']['role_name'];
                $userRole[0]['user_uuid'] = $model->attributes['user_uuid'];
                $userRole[0]['role_uuid'] = $roleId;
                $userRole[0]['role_name'] = $roleName;
                $userRole = json_encode($userRole);
                $params['user_role'] = $userRole;
                $userRolePermission['role_permission'] = array();
                $userRolePermission['institute_module_permission'] = array();
                $userRolePermission['user_grant_permission'] = array();
                $userRolePermission['user_denied_permission'] = array();
                $userRolePermission['institute_id'] = $_POST['SsoUserInformation']['institute_id'];
                $userRolePermission = json_encode($userRolePermission);
                $params['user_role_permission'] = $userRolePermission;
                $userrole = $this->mapUserRole($params);
               
                if($userrole == true){
                    $transaction->commit();
                } else {
                  $transaction->rollBack();
                }
              } else{
                $transaction->rollBack();
              }

              $response = array();
              $response['user_uuid'] = $model->attributes['user_uuid'];
              $response['user_mobile'] = $model->attributes['user_mobile'];
              $response['user_email'] = $model->attributes['user_email'];

                if(LOGIN_AFTER_REGISTRATION === TRUE){
                    $service = new SsoUserTokenService();
                    $token = $service->saveUserToken($model->attributes['user_uuid']);
                    $response['token'] = $token;
                }
               echo json_encode(array("response" => $response));
               exit;
            }else{
              echo json_encode(array("error" => $model->errors));
              exit;
            }
        }else{
           
            echo json_encode(array("error" => $_POST));
            exit;
        }
      

    }

}
